// Beam_lv1_Fire.vert

#define UWrapNum   1.0
#define VWrapNum   1.0

#define Si 1.0
//#define lineLength 3.0
#define lineLength 0.125
#define timeLength 0.25

uniform float time;
uniform vec3  cameraPosition;
uniform vec3   modelPosition;
uniform vec3   modelDirection;
uniform vec3   modelUpper;

varying vec2  texCoord;

float inv_pow(float f, float p){
	return 1.0-(pow(1.0-f, p));
}

void main(void){
	if(time < 0.0 || time > timeLength) {
		gl_Position = 0.0;
		texCoord = 0.0;
		return;
	}
	vec4  position = vec4(0.0, 0.0, 0.0, 1.0);
	vec3 direction = vec3(0.0, 0.0, -1.0); // vec

	vec3 eyevec    = normalize( modelPosition - cameraPosition);

	vec3 modelRight = normalize(cross(modelDirection, modelUpper));
	modelUpper = normalize(cross(modelRight, modelDirection));
	mat3 worldToLocal = inverse(mat3(modelRight, modelUpper, -modelDirection));
	vec3 side      = normalize(cross(eyevec, modelDirection));
	side     = worldToLocal * side;
	side.x   = -side.x;

	side *= 3.0*Si*0.1*2.0*(1.0-time);

	float len = inv_pow(time, 16.0) * 512.0 + time * 128.0;
	len  *= Si*0.1;

	direction *=  lineLength * len;

	if(gl_Vertex.z > 0.0){
		texCoord.s = 0.0; 
		position += vec4(direction, 0.0);
	}else{
		texCoord.s = 1.0 * UWrapNum;
	}

	if(gl_Vertex.x > 0.0){
		texCoord.t = 0.0;
		position += vec4( side, 0.0);
	}else{
		texCoord.t = 1.0 * VWrapNum; 
		position -= vec4( side, 0);
	}

//	gl_Position =                                position;
//	gl_Position = gl_ModelViewMatrix           * gl_Position;
	gl_Position = gl_ProjectionMatrix          * gl_Position;
}
