//@see http://www.nicovideo.jp/watch/sm22353926
// Put particle.x and particle.png in same directory(folder).

#define PARTICLE_COUNT  100

#define particleSpeed          1.0
#define particleSystemShape    1.0
#define particleSpread        10.0
#define particleSystemHeight   0.05
#define particleSize           0.1

#define GlobalRotateSpd        0.0
#define MoveSpdMin             1.0
#define MoveSpdMax             2.0
#define Air                    1.0

//#define MAX           2147483647
#define MAX           65536
#define PI                     3.14159265

vec3 Gravity = {0.0, 3.0, 0.0};
vec3 offset  = {0.0, 0.0, 1.0};

uniform float time;

//varying vec2  texCoord;
varying float color;

/**
 * @return [0.0, 1.0)
 */
vec3 getRandom(float index) {
	vec3 random;
	int i = int(index);
	random.x = ((i<<2)+1)*(i+1) % MAX;//non linear congruential method
	int x = int(random.x);
	random.y = ((x<<2)+1)*(x+1) % MAX;//non linear congruential method
	int y = int(random.y);
	random.z = ((y<<2)+1)*(y+1) % MAX;//non linear congruential method
	random    /= float(MAX);
	return random;
}

void main(void){
	float idx = round(gl_Vertex.z*float(PARTICLE_COUNT));
//	float idx = gl_VertexID;
	idx *= 10.0;
	idx += gl_Vertex.x + 3.0 * gl_Vertex.y;

	// Loop particles
	float t = fract(gl_Vertex.z + particleSpeed * time);

	// Determine the shape of the system
	float shape = pow(t, particleSystemShape);

	vec3 pos;

	pos = getRandom(idx*123.0);
	float rad  =     pos.x        *2.0*PI;
	float len  =     pos.y;
	float rnd  =     pos.z;

	pos = getRandom(rnd*12345.0);
	float radx = sin(pos.x*162.0) *2.0*PI;
	float rady = cos(pos.y*257.0) *2.0*PI;
	float radz = cos(pos.z*510.0) *2.0*PI+time*GlobalRotateSpd;
//	float radx =     pos.x        *2.0*PI;
//	float rady =     pos.y        *2.0*PI+time*GlobalRotateSpd;
//	float radz =     pos.z        *2.0*PI;

	len += 0.1;
//	pos.x = (cos(rad))*pow(t,1)*len*particleSystemHeight;
//	pos.y = (sin(rad))*pow(t,1)*len*particleSystemHeight;
	pos.x = (cos(rad))*shape   *len*particleSystemHeight;
	pos.y = (sin(rad))*shape   *len*particleSystemHeight;
	pos.z = 0;

//	mat4x4 matRot;

	//rotation around X axis
	mat3x3 matRotX = {
		{1.0,       0.0,        0.0},
		{0.0,  cos(radx), sin(radx)},
		{0.0, -sin(radx), cos(radx)}};
	pos = matRotX * pos;

	//rotation around Y axis
	mat3x3 matRotY = {
		{cos(rady), 0.0, -sin(rady)},
		{      0.0, 1.0,        0.0},
		{sin(rady), 0.0,  cos(rady)}};
	pos = matRotY * pos;

	//rotation around Z axis
	mat3x3 matRotZ = {
		{ cos(radz), sin(radz), 0.0},
		{-sin(radz), cos(radz), 0.0},
		{      0.0,        0.0, 1.0}};
	pos = matRotZ * pos;

	// Spread particles
	pos *= particleSpread;

//	// Spread particles in a semi-random fashion
////	pos.x = particleSpread * s * cos( 62.0 * idx);//square
//	pos.x = particleSpread * s * cos(163.0 * idx);//circle
//	pos.y = particleSpread * s * sin(163.0 * idx);
//	// Particles goes up(back)
//	pos.z = particleSystemHeight * t;

	float  base_alpha = 1.0;

	pos.z += pow(t,Air)*(MoveSpdMin+(MoveSpdMax-MoveSpdMin)*base_alpha);

	gl_Position.xyz  = pos + offset;
	gl_Position.w    = gl_Vertex.w;

	gl_Position = gl_ModelViewProjectionMatrix * gl_Position;
	gl_Position.xyz += pow(t, 2.0)*Gravity*0.1;

	gl_PointSize  = gl_Point.size * particleSize / gl_Position.w;

	color    = 1.0 - t;
}
